=== modified file 'NEWS'
--- NEWS	2009-04-16 21:49:48 +0000
+++ NEWS	2009-05-04 20:29:13 +0000
@@ -49,6 +49,9 @@
     - Fix bug #361238 (serve-branches was calling config.get_option()
       incorrectly and failing). (Matt Nordhoff)
 
+    - Use transport API internally, so it is possible to specify a remote 
+      URL to serve-branches. (Jelmer Vernooij, #371787)
+
 1.10 [22Dec2008]
 ---------------
 

=== modified file '__init__.py'
--- __init__.py	2009-03-10 01:01:04 +0000
+++ __init__.py	2009-05-13 14:03:48 +0000
@@ -62,7 +62,7 @@
         takes_options = _original_command.takes_options + [
             Option('http',
                 help='Run an http (Loggerhead) server to browse code, '
-                    'by default on port %s.' % DEFAULT_PORT)]
+                    'by default on port %s.' % (DEFAULT_PORT,))]
 
         def run(self, *args, **kw):
             if 'http' in kw:
@@ -71,10 +71,10 @@
                 import os.path, sys
                 sys.path.append(os.path.dirname(__file__))
 
-                from loggerhead.apps.filesystem import BranchesFromFileSystemRoot
+                from loggerhead.apps.transport import BranchesFromTransportRoot
                 from paste.httpexceptions import HTTPExceptionHandler
                 from paste.httpserver import serve
-                a = HTTPExceptionHandler(BranchesFromFileSystemRoot('.'))
+                a = HTTPExceptionHandler(BranchesFromTransportRoot('.'))
                 port = kw.get('port', DEFAULT_PORT)
                 # port might be an int already...
                 if isinstance(port, basestring) and ':' in port:

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2009-04-07 18:29:10 +0000
+++ loggerhead/apps/branch.py	2009-05-13 14:03:48 +0000
@@ -36,29 +36,33 @@
         self._config = config
         self.friendly_name = friendly_name
         self.branch_link = branch_link  # Currently only used in Launchpad
-        self.log = logging.getLogger('loggerhead.%s' % friendly_name)
+        self.log = logging.getLogger('loggerhead.%s' % (friendly_name,))
         if graph_cache is None:
-            graph_cache = bzrlib.lru_cache.LRUCache()
+            graph_cache = bzrlib.lru_cache.LRUCache(10)
         self.graph_cache = graph_cache
         self.is_root = is_root
         self.served_url = served_url
         self.use_cdn = use_cdn
 
     def get_history(self):
-        _history = History(self.branch, self.graph_cache)
+        file_cache = None
+        revinfo_disk_cache = None
         cache_path = self._config.get('cachepath', None)
         if cache_path is not None:
             # Only import the cache if we're going to use it.
             # This makes sqlite optional
             try:
-                from loggerhead.changecache import FileChangeCache
+                from loggerhead.changecache import (
+                    FileChangeCache, RevInfoDiskCache)
             except ImportError:
                 self.log.debug("Couldn't load python-sqlite,"
                                " continuing without using a cache")
             else:
-                _history.use_file_cache(
-                    FileChangeCache(_history, cache_path))
-        return _history
+                file_cache = FileChangeCache(cache_path)
+                revinfo_disk_cache = RevInfoDiskCache(cache_path)
+        return History(
+            self.branch, self.graph_cache, file_cache=file_cache,
+            revinfo_disk_cache=revinfo_disk_cache, cache_key=self.friendly_name)
 
     def url(self, *args, **kw):
         if isinstance(args[0], list):
@@ -66,7 +70,7 @@
         qs = []
         for k, v in kw.iteritems():
             if v is not None:
-                qs.append('%s=%s'%(k, urllib.quote(v)))
+                qs.append('%s=%s' % (k, urllib.quote(v)))
         qs = '&'.join(qs)
         return request.construct_url(
             self._environ, script_name=self._url_base,
@@ -115,7 +119,11 @@
             self._static_url_base = self._url_base
         self._environ = environ
         if self.served_url is _DEFAULT:
-            self.served_url = self.url([])
+            public_branch = self.branch_url()
+            if public_branch is not None:
+                self.served_url = public_branch
+            else:
+                self.served_url = self.url([])
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(

=== renamed file 'loggerhead/apps/filesystem.py' => 'loggerhead/apps/transport.py'
--- loggerhead/apps/filesystem.py	2009-04-16 23:47:26 +0000
+++ loggerhead/apps/transport.py	2009-05-13 12:00:14 +0000
@@ -1,8 +1,6 @@
-"""Serve branches at urls that mimic the file system layout."""
-
-import os
-
-from bzrlib import branch, errors, lru_cache
+"""Serve branches at urls that mimic a transport's file system layout."""
+
+from bzrlib import branch, errors, lru_cache, urlutils
 
 from paste.request import path_info_pop
 from paste import httpexceptions
@@ -14,26 +12,29 @@
 from loggerhead.controllers.directory_ui import DirectoryUI
 
 
-class BranchesFromFileSystemServer(object):
+class BranchesFromTransportServer(object):
 
-    def __init__(self, path, root, name=None):
-        self.path = path
+    def __init__(self, transport, root, name=None):
+        self.transport = transport
         self.root = root
         self.name = name
-        self._config = LoggerheadConfig()
+        self._config = root._config
 
     def app_for_branch(self, branch):
         if not self.name:
-            name = branch.get_config().get_nickname()
+            name = branch._get_nick(local=True)
             is_root = True
         else:
             name = self.name
             is_root = False
         branch_app = BranchWSGIApp(
-            branch, name,
+            branch,
+            name,
             {'cachepath': self._config.SQL_DIR},
-            self.root.graph_cache, is_root=is_root,
-            use_cdn=self._config.get_option('use_cdn'))
+            self.root.graph_cache,
+            is_root=is_root,
+            use_cdn=self._config.get_option('use_cdn'),
+            )
         return branch_app.app
 
     def app_for_non_branch(self, environ):
@@ -47,32 +48,33 @@
             else:
                 name = '/'
             return DirectoryUI(environ['loggerhead.static.url'],
-                               self.path,
+                               self.transport,
                                name)
         else:
-            new_path = os.path.join(self.path, segment)
+            new_transport = self.transport.clone(segment)
             if self.name:
-                new_name = os.path.join(self.name, segment)
+                new_name = urlutils.join(self.name, segment)
             else:
                 new_name = '/' + segment
-            return BranchesFromFileSystemServer(new_path, self.root, new_name)
+            return BranchesFromTransportServer(new_transport, self.root, new_name)
 
     def __call__(self, environ, start_response):
-        if not os.path.isdir(self.path):
-            raise httpexceptions.HTTPNotFound()
         try:
-            b = branch.Branch.open(self.path)
+            b = branch.Branch.open_from_transport(self.transport)
         except errors.NotBranchError:
+            if not self.transport.listable() or not self.transport.has('.'):
+                raise httpexceptions.HTTPNotFound()
             return self.app_for_non_branch(environ)(environ, start_response)
         else:
             return self.app_for_branch(b)(environ, start_response)
 
 
-class BranchesFromFileSystemRoot(object):
+class BranchesFromTransportRoot(object):
 
-    def __init__(self, folder):
-        self.graph_cache = lru_cache.LRUCache()
-        self.folder = folder
+    def __init__(self, transport, config):
+        self.graph_cache = lru_cache.LRUCache(10)
+        self.transport = transport
+        self._config = config
 
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
@@ -83,19 +85,20 @@
         elif environ['PATH_INFO'] == '/favicon.ico':
             return favicon_app(environ, start_response)
         elif '/.bzr/' in environ['PATH_INFO']:
-            app = urlparser.make_static(None, self.folder)
+            app = urlparser.make_static(None, self.transport)
             return app(environ, start_response)
         else:
-            return BranchesFromFileSystemServer(
-                self.folder, self)(environ, start_response)
-
-
-class UserBranchesFromFileSystemRoot(object):
-
-    def __init__(self, folder, trunk_dir):
-        self.graph_cache = lru_cache.LRUCache()
-        self.folder = folder
-        self.trunk_dir = trunk_dir
+            return BranchesFromTransportServer(
+                self.transport, self)(environ, start_response)
+
+
+class UserBranchesFromTransportRoot(object):
+
+    def __init__(self, transport, config):
+        self.graph_cache = lru_cache.LRUCache(10)
+        self.transport = transport
+        self._config = config
+        self.trunk_dir = config.get_option('trunk_dir')
 
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
@@ -110,10 +113,10 @@
             # segments starting with ~ are user branches
             if path_info.startswith('/~'):
                 segment = path_info_pop(environ)
-                new_path = os.path.join(self.folder, segment[1:])
-                return BranchesFromFileSystemServer(
-                    new_path, self, segment)(environ, start_response)
+                new_transport = self.transport.clone(segment[1:])
+                return BranchesFromTransportServer(
+                    new_transport, self, segment)(environ, start_response)
             else:
-                new_path = os.path.join(self.folder, self.trunk_dir)
-                return BranchesFromFileSystemServer(
-                    new_path, self)(environ, start_response)
+                new_transport = self.transport.clone(self.trunk_dir)
+                return BranchesFromTransportServer(
+                    new_transport, self)(environ, start_response)

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2009-03-19 21:01:11 +0000
+++ loggerhead/changecache.py	2009-05-02 14:01:05 +0000
@@ -27,8 +27,10 @@
 """
 
 import cPickle
+import marshal
 import os
 import tempfile
+import zlib
 
 try:
     from sqlite3 import dbapi2
@@ -38,16 +40,26 @@
 # We take an optimistic approach to concurrency here: we might do work twice
 # in the case of races, but not crash or corrupt data.
 
+def safe_init_db(filename, init_sql):
+    # To avoid races around creating the database, we create the db in
+    # a temporary file and rename it into the ultimate location.
+    fd, temp_path = tempfile.mkstemp(dir=os.path.dirname(filename))
+    os.close(fd)
+    con = dbapi2.connect(temp_path)
+    cur = con.cursor()
+    cur.execute(init_sql)
+    con.commit()
+    con.close()
+    os.rename(temp_path, filename)
+
 class FakeShelf(object):
 
     def __init__(self, filename):
         create_table = not os.path.exists(filename)
         if create_table:
-            # To avoid races around creating the database, we create the db in
-            # a temporary file and rename it into the ultimate location.
-            fd, path = tempfile.mkstemp(dir=os.path.dirname(filename))
-            self._create_table(path)
-            os.rename(path, filename)
+            safe_init_db(
+                filename, "create table RevisionData "
+                "(revid binary primary key, data binary)")
         self.connection = dbapi2.connect(filename)
         self.cursor = self.connection.cursor()
 
@@ -89,8 +101,7 @@
 
 class FileChangeCache(object):
 
-    def __init__(self, history, cache_path):
-        self.history = history
+    def __init__(self, cache_path):
 
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
@@ -104,3 +115,44 @@
             changes = self.history.get_file_changes_uncached(entry)
             cache.add(entry.revid, changes)
         return changes
+
+
+class RevInfoDiskCache(object):
+    """Like `RevInfoMemoryCache` but backed in a sqlite DB."""
+
+    def __init__(self, cache_path):
+        if not os.path.exists(cache_path):
+            os.mkdir(cache_path)
+        filename = os.path.join(cache_path, 'revinfo.sql')
+        create_table = not os.path.exists(filename)
+        if create_table:
+            safe_init_db(
+                filename, "create table Data "
+                "(key binary primary key, revid binary, data binary)")
+        self.connection = dbapi2.connect(filename)
+        self.cursor = self.connection.cursor()
+
+    def get(self, key, revid):
+        self.cursor.execute(
+            "select revid, data from data where key = ?", (dbapi2.Binary(key),))
+        row = self.cursor.fetchone()
+        if row is None:
+            return None
+        elif str(row[0]) != revid:
+            return None
+        else:
+            return marshal.loads(zlib.decompress(row[1]))
+
+    def set(self, key, revid, data):
+        try:
+            self.cursor.execute(
+                'delete from data where key = ?', (dbapi2.Binary(key), ))
+            blob = zlib.compress(marshal.dumps(data))
+            self.cursor.execute(
+                "insert into data (key, revid, data) values (?, ?, ?)",
+                map(dbapi2.Binary, [key, revid, blob]))
+            self.connection.commit()
+        except dbapi2.IntegrityError:
+            # If another thread or process attempted to set the same key, we
+            # don't care too much -- it's only a cache after all!
+            pass

=== modified file 'loggerhead/config.py'
--- loggerhead/config.py	2009-04-07 19:00:26 +0000
+++ loggerhead/config.py	2009-04-30 10:27:43 +0000
@@ -4,6 +4,14 @@
 import tempfile
 
 
+_temporary_sql_dir = None
+
+def _get_temporary_sql_dir():
+    global _temporary_sql_dir
+    if _temporary_sql_dir is None:
+        _temporary_sql_dir = tempfile.mkdtemp(prefix='loggerhead-cache-')
+    return _temporary_sql_dir
+
 def command_line_parser():
     parser = OptionParser("%prog [options] <path>")
     parser.set_defaults(
@@ -11,8 +19,9 @@
         show_version=False,
         log_folder=None,
         use_cdn=False,
+        sql_dir=None,
         )
-    parser.add_option("--user-dirs", action="store_true", dest="user_dirs",
+    parser.add_option("--user-dirs", action="store_true",
                       help="Serve user directories as ~user.")
     parser.add_option("--trunk-dir", metavar="DIR",
                       help="The directory that contains the trunk branches.")
@@ -22,22 +31,23 @@
     parser.add_option("--host", dest="user_host",
                       help="Host Loggerhead should listen on.")
     parser.add_option('--memory-profile', action='store_true',
-                      dest='memory_profile',
                       help='Profile the memory usage using heapy.')
     parser.add_option("--prefix", dest="user_prefix",
                       help="Specify host prefix.")
-    parser.add_option("--profile", action="store_true", dest="profile",
+    parser.add_option("--profile", action="store_true",
                       help="Generate callgrind profile data to "
                         "%d-stats.callgrind on each request.")
-    parser.add_option("--reload", action="store_true", dest="reload",
+    parser.add_option("--reload", action="store_true",
                       help="Restarts the application when changing python"
                            " files. Only used for development purposes.")
-    parser.add_option('--log-folder', dest="log_folder",
-                      type=str, help="The directory to place log files in.")
+    parser.add_option('--log-folder',
+                      help="The directory to place log files in.")
     parser.add_option("--version", action="store_true", dest="show_version",
                       help="Print the software version and exit")
     parser.add_option('--use-cdn', action='store_true',
                       help="Serve YUI from Yahoo!'s CDN")
+    parser.add_option('--cache-dir', dest='sql_dir',
+                      help="The directory to place the SQL cache in")
     return parser
 
 
@@ -48,7 +58,10 @@
         self._parser = command_line_parser()
         self._options, self._args = self._parser.parse_args(sys.argv[1:])
 
-        self.SQL_DIR = tempfile.mkdtemp(prefix='loggerhead-cache-')
+        sql_dir = self.get_option('sql_dir')
+        if sql_dir is None:
+            sql_dir = _get_temporary_sql_dir()
+        self.SQL_DIR = sql_dir
 
     def get_option(self, option):
         '''Get an option from the options dict.'''
@@ -66,4 +79,3 @@
     def arg_count(self):
         '''Return the number of args from the option parser.'''
         return len(self._args)
-

=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2009-03-18 19:34:07 +0000
+++ loggerhead/controllers/annotate_ui.py	2009-05-13 14:03:48 +0000
@@ -85,7 +85,7 @@
                     change=change, text=hl_lines[lineno - 1])
                 lineno += 1
 
-        self.log.debug('annotate: %r secs' % (time.time() - z))
+        self.log.debug('annotate: %r secs' % (time.time() - z,))
 
     def get_values(self, path, kwargs, headers):
         history = self._history

=== modified file 'loggerhead/controllers/diff_ui.py'
--- loggerhead/controllers/diff_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/diff_ui.py	2009-05-13 14:03:48 +0000
@@ -73,7 +73,7 @@
         headers = [
             ('Content-Type', 'application/octet-stream'),
             ('Content-Length', len(content)),
-            ('Content-Disposition', 'attachment; filename=%s' % filename),
+            ('Content-Disposition', 'attachment; filename=%s' % (filename,)),
             ]
         start_response('200 OK', headers)
         return [content]

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/directory_ui.py	2009-05-05 18:01:25 +0000
@@ -18,7 +18,7 @@
 
 import datetime
 import logging
-import os
+import stat
 
 from bzrlib import branch
 
@@ -48,7 +48,7 @@
 
     template_path = 'loggerhead.templates.directory'
 
-    def __init__(self, static_url_base, path, name):
+    def __init__(self, static_url_base, transport, name):
 
         class _branch(object):
             context_url = 1
@@ -57,23 +57,22 @@
             def static_url(path):
                 return self._static_url_base + path
         self._branch = _branch
-        self._history_callable = lambda:None
-        self._path = path
+        self._history_callable = lambda: None
         self._name = name
         self._static_url_base = static_url_base
+        self.transport = transport
         self.log = logging.getLogger('')
 
     def get_values(self, path, kwargs, response):
-        listing = [d for d in os.listdir(self._path)
+        listing = [d for d in self.transport.list_dir('.')
                    if not d.startswith('.')
-                   and os.path.isdir(os.path.join(self._path, d))]
+                   and stat.S_ISDIR(self.transport.stat(d).st_mode)]
         listing.sort(key=lambda x: x.lower())
         dirs = []
         parity = 0
         for d in listing:
-            p = os.path.join(self._path, d)
             try:
-                b = branch.Branch.open(p)
+                b = branch.Branch.open_from_transport(self.transport.clone(d))
             except:
                 b = None
             dirs.append(DirEntry(d, parity, b))

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2009-02-06 05:53:36 +0000
+++ loggerhead/controllers/download_ui.py	2009-05-13 14:03:48 +0000
@@ -65,7 +65,7 @@
             ('Content-Type', mime_type),
             ('Content-Length', len(content)),
             ('Content-Disposition',
-             'attachment; filename*=utf-8\'\'%s' % encoded_filename),
+             'attachment; filename*=utf-8\'\'%s' % (encoded_filename,)),
             ]
         start_response('200 OK', headers)
         return [content]

=== modified file 'loggerhead/daemon.py'
--- loggerhead/daemon.py	2008-12-07 15:24:05 +0000
+++ loggerhead/daemon.py	2009-05-13 14:03:48 +0000
@@ -53,7 +53,7 @@
             pass
 
     f = open(pidfile, 'w')
-    f.write('%d\n' % os.getpid())
+    f.write('%d\n' % (os.getpid(),))
     f.close()
 
 

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2009-04-18 06:16:28 +0000
+++ loggerhead/history.py	2009-05-13 14:03:48 +0000
@@ -31,9 +31,11 @@
 import bisect
 import datetime
 import logging
+import marshal
 import re
 import textwrap
 import threading
+from StringIO import StringIO
 
 from loggerhead import search
 from loggerhead import util
@@ -42,6 +44,7 @@
 import bzrlib.branch
 import bzrlib.delta
 import bzrlib.errors
+import bzrlib.lru_cache
 import bzrlib.progress
 import bzrlib.revision
 import bzrlib.ui
@@ -171,6 +174,43 @@
                 file_id=file_id))
 
 
+class RevInfoMemoryCache(object):
+    """A store that validates values against the revids they were stored with.
+
+    We use a unique key for each branch.
+
+    The reason for not just using the revid as the key is so that when a new
+    value is provided for a branch, we replace the old value used for the
+    branch.
+
+    There is another implementation of the same interface in
+    loggerhead.changecache.RevInfoDiskCache.
+    """
+
+    def __init__(self, cache):
+        self._cache = cache
+
+    def get(self, key, revid):
+        """Return the data associated with `key`, subject to a revid check.
+
+        If a value was stored under `key`, with the same revid, return it.
+        Otherwise return None.
+        """
+        cached = self._cache.get(key)
+        if cached is None:
+            return None
+        stored_revid, data = cached
+        if revid == stored_revid:
+            return data
+        else:
+            return None
+
+    def set(self, key, revid, data):
+        """Store `data` under `key`, to be checked against `revid` on get().
+        """
+        self._cache[key] = (revid, data)
+
+
 class History(object):
     """Decorate a branch to provide information for rendering.
 
@@ -179,31 +219,87 @@
     around it, serve the request, throw the History object away, unlock the
     branch and throw it away.
 
-    :ivar _file_change_cache: xx
+    :ivar _file_change_cache: An object that caches information about the
+        files that changed between two revisions.
+    :ivar _rev_info: A list of information about revisions.  This is by far
+        the most cryptic data structure in loggerhead.  At the top level, it
+        is a list of 3-tuples [(merge-info, where-merged, parents)].
+        `merge-info` is (seq, revid, merge_depth, revno_str, end_of_merge) --
+        like a merged sorted list, but the revno is stringified.
+        `where-merged` is a tuple of revisions that have this revision as a
+        non-lefthand parent.  Finally, `parents` is just the usual list of
+        parents of this revision.
+    :ivar _rev_indices: A dictionary mapping each revision id to the index of
+        the information about it in _rev_info.
+    :ivar _full_history: A list of all revision ids in the ancestry of the
+        branch, in merge-sorted order.  This is a bit silly, and shouldn't
+        really be stored on the instance...
+    :ivar _revno_revid: A dictionary mapping stringified revnos to revision
+        ids.
     """
 
-    def __init__(self, branch, whole_history_data_cache):
+    def _load_whole_history_data(self, caches, cache_key):
+        """Set the attributes relating to the whole history of the branch.
+
+        :param caches: a list of caches with interfaces like
+            `RevInfoMemoryCache` and be ordered from fastest to slowest.
+        :param cache_key: the key to use with the caches.
+        """
+        self._rev_indices = None
+        self._rev_info = None
+
+        missed_caches = []
+        def update_missed_caches():
+            for cache in missed_caches:
+                cache.set(cache_key, self.last_revid, self._rev_info)
+        for cache in caches:
+            data = cache.get(cache_key, self.last_revid)
+            if data is not None:
+                self._rev_info = data
+                update_missed_caches()
+                break
+            else:
+                missed_caches.append(cache)
+        else:
+            whole_history_data = compute_whole_history_data(self._branch)
+            self._rev_info, self._rev_indices = whole_history_data
+            update_missed_caches()
+
+        if self._rev_indices is not None:
+            self._full_history = []
+            self._revno_revid = {}
+            for ((_, revid, _, revno_str, _), _, _) in self._rev_info:
+                self._revno_revid[revno_str] = revid
+                self._full_history.append(revid)
+        else:
+            self._full_history = []
+            self._revno_revid = {}
+            self._rev_indices = {}
+            for ((seq, revid, _, revno_str, _), _, _) in self._rev_info:
+                self._rev_indices[revid] = seq
+                self._revno_revid[revno_str] = revid
+                self._full_history.append(revid)
+
+    def __init__(self, branch, whole_history_data_cache, file_cache=None,
+                 revinfo_disk_cache=None, cache_key=None):
         assert branch.is_locked(), (
             "Can only construct a History object with a read-locked branch.")
-        self._file_change_cache = None
+        if file_cache is not None:
+            self._file_change_cache = file_cache
+            file_cache.history = self
+        else:
+            self._file_change_cache = None
         self._branch = branch
         self._inventory_cache = {}
         self._branch_nick = self._branch.get_config().get_nickname()
-        self.log = logging.getLogger('loggerhead.%s' % self._branch_nick)
+        self.log = logging.getLogger('loggerhead.%s' % (self._branch_nick,))
 
         self.last_revid = branch.last_revision()
 
-        whole_history_data = whole_history_data_cache.get(self.last_revid)
-        if whole_history_data is None:
-            whole_history_data = compute_whole_history_data(branch)
-            whole_history_data_cache[self.last_revid] = whole_history_data
-
-        (self._revision_graph, self._full_history, self._revision_info,
-         self._revno_revid, self._merge_sort, self._where_merged,
-         ) = whole_history_data
-
-    def use_file_cache(self, cache):
-        self._file_change_cache = cache
+        caches = [RevInfoMemoryCache(whole_history_data_cache)]
+        if revinfo_disk_cache:
+            caches.append(revinfo_disk_cache)
+        self._load_whole_history_data(caches, cache_key)
 
     @property
     def has_revisions(self):
@@ -213,12 +309,12 @@
         return self._branch.get_config()
 
     def get_revno(self, revid):
-        if revid not in self._revision_info:
+        if revid not in self._rev_indices:
             # ghost parent?
             return 'unknown'
-        (seq, revid, merge_depth,
-         revno_str, end_of_merge) = self._revision_info[revid]
-        return revno_str
+        seq = self._rev_indices[revid]
+        revno = self._rev_info[seq][0][3]
+        return revno
 
     def get_revids_from(self, revid_list, start_revid):
         """
@@ -232,10 +328,11 @@
 
         def introduced_revisions(revid):
             r = set([revid])
-            seq, revid, md, revno, end_of_merge = self._revision_info[revid]
+            seq = self._rev_indices[revid]
+            md = self._rev_info[seq][0][2]
             i = seq + 1
-            while i < len(self._merge_sort) and self._merge_sort[i][2] > md:
-                r.add(self._merge_sort[i][1])
+            while i < len(self._rev_info) and self._rev_info[i][0][2] > md:
+                r.add(self._rev_info[i][0][1])
                 i += 1
             return r
         while 1:
@@ -243,7 +340,7 @@
                 return
             if introduced_revisions(revid) & revid_set:
                 yield revid
-            parents = self._revision_graph[revid]
+            parents = self._rev_info[self._rev_indices[revid]][2]
             if len(parents) == 0:
                 return
             revid = parents[0]
@@ -460,10 +557,10 @@
 
         merge_point = []
         while True:
-            children = self._where_merged.get(revid, [])
+            children = self._rev_info[self._rev_indices[revid]][1]
             nexts = []
             for child in children:
-                child_parents = self._revision_graph[child]
+                child_parents = self._rev_info[self._rev_indices[child]][2]
                 if child_parents[0] == revid:
                     nexts.append(child)
                 else:

=== modified file 'loggerhead/middleware/profile.py'
--- loggerhead/middleware/profile.py	2009-03-30 22:05:16 +0000
+++ loggerhead/middleware/profile.py	2009-04-30 10:27:53 +0000
@@ -1,11 +1,11 @@
 '''Profiling middleware for paste.'''
-import cgi
+
 import logging
 import sys
 import threading
 
 from bzrlib.lsprof import profile
-from guppy import hpy
+
 
 class LSProfMiddleware(object):
     '''Paste middleware for profiling with lsprof.'''
@@ -33,77 +33,3 @@
             return ret
         finally:
             self.lock.release()
-
-
-class MemoryProfileMiddleware(object):
-    '''Paste middleware for profiling memory with heapy.'''
-
-    def __init__(self, app, limit=40):
-        self.app = app
-        self.lock = threading.Lock()
-
-        self.type2count = {}
-        self.type2all = {}
-        self.limit = limit
-
-    def update(self):
-        try:
-            obs = sys.getobjects(0)
-        except AttributeError:
-            logging.error(
-                'Python does not have debug symbols compiled.  Memory will '
-                'not be profiled.')
-            return
-        type2count = {}
-        type2all = {}
-        for o in obs:
-            all = sys.getrefcount(o)
-
-            if type(o) is str and o == '<dummy key>':
-                # avoid dictionary madness
-                continue
-            t = type(o)
-            if t in type2count:
-                type2count[t] += 1
-                type2all[t] += all
-            else:
-                type2count[t] = 1
-                type2all[t] = all
-
-        ct = [(type2count[t] - self.type2count.get(t, 0),
-               type2all[t] - self.type2all.get(t, 0),
-               t)
-              for t in type2count.iterkeys()]
-        ct.sort()
-        ct.reverse()
-        printed = False
-
-        logger = logging.getLogger('loggerhead-memprofile')
-        logger.debug('*' * 20)
-        logger.debug("Loggerhead Memory Profiling")
-        i = 0
-        for delta1, delta2, t in ct:
-            if delta1 or delta2:
-                if not printed:
-                    logger.debug("%-55s %8s %8s" % ('', 'insts', 'refs'))
-                    printed = True
-
-                logger.debug("%-55s %8d %8d" % (t, delta1, delta2))
-
-                i += 1
-                if i >= self.limit:
-                    break
-
-        self.type2count = type2count
-        self.type2all = type2all
-
-    def __call__(self, environ, start_response):
-        self.lock.acquire()
-        try:
-            # We don't want to be working with the static images
-            # TODO: There needs to be a better way to do this.
-            self.update()
-            return self.app(environ, start_response)
-        finally:
-            self.lock.release()
-

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2009-04-02 20:32:12 +0000
+++ loggerhead/util.py	2009-05-13 14:03:48 +0000
@@ -129,7 +129,7 @@
     return _wrap_with_date_time_title(date, _displaydate(date))
 
 
-class Container (object):
+class Container(object):
     """
     Convert a dict into an object with attributes.
     """
@@ -316,7 +316,7 @@
 
     out = str(base)
     if (base < 100) and (dot != 0):
-        out += '.%d' % (dot)
+        out += '.%d' % (dot,)
     if divisor == KILO:
         out += 'K'
     elif divisor == MEG:
@@ -465,7 +465,7 @@
         now = time.time()
         msec = int(now * 1000) % 1000
         timestr = time.strftime('%Y%m%d%H%M%S',
-                                time.localtime(now)) + ('%03d' % msec)
+                                time.localtime(now)) + ('%03d' % (msec,))
         filename = f.__name__ + '-' + timestr + '.lsprof'
         cPickle.dump(stats, open(filename, 'w'), 2)
         return ret
@@ -537,7 +537,7 @@
     _reloader_environ_key = 'PYTHON_RELOADER_SHOULD_RUN'
 
     @classmethod
-    def _turn_sigterm_into_systemexit(self):
+    def _turn_sigterm_into_systemexit(cls):
         """
         Attempts to turn a SIGTERM exception into a SystemExit exception.
         """
@@ -551,26 +551,26 @@
         signal.signal(signal.SIGTERM, handle_term)
 
     @classmethod
-    def is_installed(self):
-        return os.environ.get(self._reloader_environ_key)
+    def is_installed(cls):
+        return os.environ.get(cls._reloader_environ_key)
 
     @classmethod
-    def install(self):
+    def install(cls):
         from paste import reloader
         reloader.install(int(1))
 
     @classmethod
-    def restart_with_reloader(self):
+    def restart_with_reloader(cls):
         """Based on restart_with_monitor from paste.script.serve."""
         print 'Starting subprocess with file monitor'
-        while 1:
+        while True:
             args = [sys.executable] + sys.argv
             new_environ = os.environ.copy()
-            new_environ[self._reloader_environ_key] = 'true'
+            new_environ[cls._reloader_environ_key] = 'true'
             proc = None
             try:
                 try:
-                    self._turn_sigterm_into_systemexit()
+                    cls._turn_sigterm_into_systemexit()
                     proc = subprocess.Popen(args, env=new_environ)
                     exit_code = proc.wait()
                     proc = None
@@ -579,7 +579,7 @@
                     return 1
             finally:
                 if (proc is not None
-                    and hasattr(os, 'kill')):
+                    and getattr(os, 'kill', None) is not None):
                     import signal
                     try:
                         os.kill(proc.pid, signal.SIGTERM)

=== modified file 'loggerhead/wholehistory.py'
--- loggerhead/wholehistory.py	2008-12-05 20:09:14 +0000
+++ loggerhead/wholehistory.py	2009-05-13 14:03:48 +0000
@@ -37,12 +37,16 @@
 
 
 def compute_whole_history_data(branch):
+    """Compute _rev_info and _rev_indices for a branch.
+
+    See History.__doc__ for what these data structures mean.
+    """
     z = time.time()
 
     last_revid = branch.last_revision()
 
     log = logging.getLogger('loggerhead.%s' %
-                            branch.get_config().get_nickname())
+                            (branch.get_config().get_nickname(),))
 
     graph = branch.repository.get_graph()
     parent_map = dict(((key, value) for key, value in
@@ -50,30 +54,32 @@
 
     _revision_graph = _strip_NULL_ghosts(parent_map)
     _full_history = []
-    _revision_info = {}
-    _revno_revid = {}
+
+    _rev_info = []
+    _rev_indices = {}
+
     if is_null(last_revid):
         _merge_sort = []
     else:
         _merge_sort = merge_sort(
             _revision_graph, last_revid, generate_revno=True)
 
-    for (seq, revid, merge_depth, revno, end_of_merge) in _merge_sort:
+    for info in _merge_sort:
+        seq, revid, merge_depth, revno, end_of_merge = info
         _full_history.append(revid)
         revno_str = '.'.join(str(n) for n in revno)
-        _revno_revid[revno_str] = revid
-        _revision_info[revid] = (
-            seq, revid, merge_depth, revno_str, end_of_merge)
-
-    _where_merged = {}
+        parents = _revision_graph[revid]
+        _rev_indices[revid] = len(_rev_info)
+        _rev_info.append([(seq, revid, merge_depth, revno_str, end_of_merge), (), parents])
 
     for revid in _revision_graph.keys():
-        if _revision_info[revid][2] == 0:
+        if _rev_info[_rev_indices[revid]][0][2] == 0:
             continue
         for parent in _revision_graph[revid]:
-            _where_merged.setdefault(parent, set()).add(revid)
+            c = _rev_info[_rev_indices[parent]]
+            if revid not in c[1]:
+                c[1] = c[1] + (revid,)
 
     log.info('built revision graph cache: %r secs' % (time.time() - z))
 
-    return (_revision_graph, _full_history, _revision_info,
-            _revno_revid, _merge_sort, _where_merged)
+    return (_rev_info, _rev_indices)

=== modified file 'serve-branches'
--- serve-branches	2009-04-16 21:49:48 +0000
+++ serve-branches	2009-05-13 14:03:48 +0000
@@ -20,14 +20,15 @@
 import sys
 
 from bzrlib.plugin import load_plugins
+from bzrlib.transport import get_transport
 
 from paste import httpserver
 from paste.httpexceptions import HTTPExceptionHandler, HTTPInternalServerError
 from paste.translogger import TransLogger
 
 from loggerhead import __version__
-from loggerhead.apps.filesystem import (
-    BranchesFromFileSystemRoot, UserBranchesFromFileSystemRoot)
+from loggerhead.apps.transport import (
+    BranchesFromTransportRoot, UserBranchesFromTransportRoot)
 from loggerhead.config import LoggerheadConfig
 from loggerhead.util import Reloader
 from loggerhead.apps.error import ErrorHandlerApp
@@ -37,7 +38,7 @@
     config = LoggerheadConfig()
 
     if config.get_option('show_version'):
-        print "loggerhead %s" % __version__
+        print "loggerhead %s" % (__version__,)
         sys.exit(0)
 
     if config.arg_count > 1:
@@ -48,9 +49,9 @@
     else:
         path = '.'
 
-    if not os.path.isdir(path):
-        print "%s is not a directory" % path
-        sys.exit(1)
+    load_plugins()
+
+    transport = get_transport(path)
 
     if config.get_option('trunk_dir') and not config.get_option('user_dirs'):
         print "--trunk-dir is only valid with --user-dirs"
@@ -66,10 +67,9 @@
         if not config.get_option('trunk_dir'):
             print "You didn't specify a directory for the trunk directories."
             sys.exit(1)
-        app = UserBranchesFromFileSystemRoot(
-            path, config.get_option('trunk_dir'))
+        app = UserBranchesFromTransportRoot(transport, config)
     else:
-        app = BranchesFromFileSystemRoot(path)
+        app = BranchesFromTransportRoot(transport, config)
 
     # setup_logging()
     logging.basicConfig()
@@ -87,11 +87,6 @@
     logfile.setLevel(logging.DEBUG)
     logger.addHandler(logfile)
 
-    if config.get_option('memory_profile'):
-        memprofile = logging.getLogger('loggerhead-memprofile')
-        memprofile.setLevel(logging.DEBUG)
-        memprofile.addHandler(logging.FileHandler('loggerhead-memprofile'))
-
     # setup_logging() #end
 
     app = TransLogger(app, logger=logger)
@@ -99,8 +94,8 @@
         from loggerhead.middleware.profile import LSProfMiddleware
         app = LSProfMiddleware(app)
     if config.get_option('memory_profile'):
-        from loggerhead.middleware.profile import MemoryProfileMiddleware
-        app = MemoryProfileMiddleware(app)
+        from dozer import Dozer
+        app = Dozer(app)
 
     if not config.get_option('user_prefix'):
         prefix = '/'
@@ -140,8 +135,6 @@
     else:
         host = config.get_option('user_host')
 
-    load_plugins()
-
     httpserver.serve(app, host=host, port=port)
 
 

=== modified file 'start-loggerhead'
--- start-loggerhead	2009-02-19 01:24:20 +0000
+++ start-loggerhead	2009-05-13 14:03:48 +0000
@@ -17,7 +17,6 @@
 
 
 import logging
-import logging.handlers
 from optparse import OptionParser
 import os
 import sys
@@ -32,7 +31,7 @@
 
 from loggerhead import daemon
 from loggerhead.apps.config import Root
-from loggerhead.trace import make_handler, setup_logging
+from loggerhead.trace import setup_logging
 from loggerhead.apps.error import ErrorHandlerApp
 
 
@@ -93,7 +92,7 @@
     if not options.foreground:
         sys.stderr.write('\n')
         sys.stderr.write('Launching loggerhead into the background.\n')
-        sys.stderr.write('PID file: %s\n' % options.pidfile)
+        sys.stderr.write('PID file: %s\n' % (options.pidfile,))
         sys.stderr.write('\n')
 
         daemon.daemonize(options.pidfile, home)

=== modified file 'stop-loggerhead'
--- stop-loggerhead	2008-10-24 02:26:05 +0000
+++ stop-loggerhead	2009-05-13 14:03:48 +0000
@@ -32,7 +32,7 @@
 try:
     f = open(options.pidfile, 'r')
 except IOError, e:
-    print 'Pid file %s not found.' % options.pidfile
+    print 'Pid file %s not found.' % (options.pidfile,)
     sys.exit(1)
 
 pid = int(f.readline())
@@ -44,7 +44,7 @@
     sys.exit(1)
 
 print
-print 'Shutting down previous server @ pid %d.' % pid
+print 'Shutting down previous server @ pid %d.' % (pid,)
 print
 
 import signal

